// Sketch shader - by guest(r)
// guest.r@gmail.com
//
// license: GNU-GPL


half threshold = 0.125; // effects drawing


// definitions and sampler init 

half4x4 World			: WORLD;
half4x4 View			: VIEW;
half4x4 Projection		: PROJECTION;
half4x4 WorldView		: WORLDVIEW;			
half4x4 ViewProjection		: VIEWPROJECTION;		
half4x4 WorldViewProjection	: WORLDVIEWPROJECTION;		

float2 ps  : TEXELSIZE = 1.0F/256.0F;


texture s_tex	: SOURCETEXTURE;
sampler	s_sam = sampler_state
{
 Texture   = (s_tex);
 MinFilter = LINEAR;
 MagFilter = LINEAR;
};


string name : NAME = "Sketch";
string combineTechique : COMBINETECHNIQUE =  "Sketch";


// vertex shader linkin' structure

struct VERTEX_STUFF1
{
  float4  coord : POSITION;
  float2  CT   : TEXCOORD0;
  float2  L    : TEXCOORD1;
  float2  R    : TEXCOORD2;
  float2  U    : TEXCOORD3;
  float2  D    : TEXCOORD4;
};


// **VS**

VERTEX_STUFF1 S_VERTEX (float3 p : POSITION, float2 tc : TEXCOORD0)
{
  VERTEX_STUFF1 OUT = (VERTEX_STUFF1)0;
  
  float dx = ps.x*0.5;
  float dy = ps.y*0.5;

  OUT.coord = mul(float4(p,1),WorldViewProjection);
  OUT.CT = tc;
  OUT.L = tc + float2(-dx, 0);
  OUT.R = tc + float2( dx, 0);
  OUT.U = tc + float2( 0,-dy);
  OUT.D = tc + float2( 0, dy);
  return OUT;
}

// **PS**

float4 S_FRAGMENT ( in VERTEX_STUFF1 VAR ) : COLOR
{
  half3 pap = half3(0.82, 0.77, 0.61);
  half3 ink = half3(0.28, 0.32, 0.32);  
  half3 c11 = tex2D(s_sam, VAR.CT).xyz;
  half3 c01 = tex2D(s_sam, VAR.L ).xyz;
  half3 c21 = tex2D(s_sam, VAR.R ).xyz;
  half3 c10 = tex2D(s_sam, VAR.U ).xyz;
  half3 c12 = tex2D(s_sam, VAR.D ).xyz;
	
  half3 dt = float3(1,1,1);
  
  half lum_ct = dot(c11,dt)*0.5 + 0.5;
  half lum_x  = dot(c01+c21,dt)*0.2 + lum_ct;
  half lum_y  = dot(c10+c12,dt)*0.2 + lum_ct;

  half d = dot(abs(c01-c21),dt)/lum_x + dot(abs(c10-c12),dt)/lum_y;
  half a = max(0.0, 3.0*d-0.15);

  return half4((1.0-a)*pap + a*ink,1);
}



technique Sketch
{
   pass P0
   {
     VertexShader = compile vs_2_0 S_VERTEX();
     PixelShader  = compile ps_2_0 S_FRAGMENT();
   }  
}
